from typing import Dict, Any
from .base import BaseClient


class AppsAPI:
    def __init__(self, client: BaseClient):
        self.client = client

    def create_app(self, name: str, origin: str) -> Dict[str, Any]:
        """
        Creates a new app using the provided name.

        :param name: App name
        :param origin: App origin
        :return: Created app data dictionary
        :raises ValueError: If name or workspace_secret is not provided
        """
        if not name:
            raise ValueError("The 'name' is required.")

        headers = {
            "Authorization": self.client.workspace_secret
        }
        path = f"/apps"
        files = {
            "name": (None, name),
            "origin": (None, origin),
            "workspace_id": (None, self.client.workspace_id),
        }  # multipart/form-data

        data = self.client.request("POST", path, headers=headers, files=files)
        return data or {}

    def get_app_by_id(self, app_id: str) -> Dict[str, Any]:
        """
        Retrieves a app by unique ID.

        :param app_id: App ID
        :return: App data dictionary
        """
        headers = {"Authorization": self.client.app_secret}
        path = f"/apps/{app_id}"
        data = self.client.request("GET", path, headers=headers)
        return data or {}

    def list_apps(self) -> Dict[str, Any]:
        """
        Lists apps.

        :return: Dictionary with apps list
        """
        headers = {"Authorization": self.client.workspace_secret}
        path = f"/apps"
        return self.client.request("GET", path, headers=headers) or {}

    def delete_app(self, app_id: str) -> bool:
        """
        Deletes the app by ID.

        :param app_id: ID of the app to delete
        :return: True if deletion was successful
        :raises ValueError: If app_id is missing
        """
        if not app_id:
            raise ValueError("The 'app_id' is required.")

        headers = {
            "Authorization": self.client.workspace_secret,
        }
        path = f"/apps/{app_id}"
        response = self.client.request("DELETE", path, headers=headers)

        return response.get("status") == "ok" or response == {}



from scute_client.apis.base import BaseClient
from scute_client.apis.auth import AuthAPI
from scute_client.apis.users import UsersAPI
from scute_client.apis.tokens import TokensAPI
from scute_client.apis.apps import AppsAPI

class ScuteClient:
    def __init__(
        self,
        workspace_id: str = "workspace_id",
        workspace_secret: str = "workspace_secret",
        app_id: str = "app_id",
        app_secret: str = "app_secret",
        base_url: str = "https://api.scute.io",
        api_version: str = "v1",
        timeout: int = 5,
    ):
        self.base = BaseClient(
            workspace_id=workspace_id,
            workspace_secret=workspace_secret,
            app_id=app_id,
            app_secret=app_secret,
            base_url=base_url,
            api_version=api_version,
            timeout=timeout
        )
        self.auth = AuthAPI(client=self.base)
        self.tokens = TokensAPI(client=self.base)
        self.users = UsersAPI(client=self.base)
        self.apps = AppsAPI(client=self.base)



from typing import Optional

import requests


class APIClientError(Exception):
    """Base exception for all client errors."""

class APIRequestError(APIClientError):
    def __init__(self, status_code: int, message: str, response: Optional[requests.Response] = None):
        super().__init__(f"API request failed with status {status_code}: {message}")
        self.status_code = status_code
        self.message = message
        self.response = response



from typing import Dict, Any, Optional, Iterator
from .base import BaseClient

class UsersAPI:
    def __init__(self, client: BaseClient):
        self.client = client

    def create_user(self, identifier: str) -> Dict[str, Any]:
        """
        Creates a user using the given identifier (email or phone).

        :param identifier: User's identifier (email or phone number)
        :return: Created user data dictionary
        :raises ValueError: If identifier is not provided
        """
        if not identifier:
            raise ValueError("The 'identifier' (email or phone) is required.")

        headers = {
            "Authorization": self.client.workspace_secret if self.client._workspace_secret else self.client.app_secret
        }
        path = f"/{self.client.app_id}/users"
        files = {"identifier": (None, identifier)}  # multipart/form-data

        data = self.client.request("POST", path, headers=headers, files=files)
        return data or {}

    def get_user_by_id(self, user_id: str) -> Dict[str, Any]:
        """
        Retrieves a user by their unique ID.

        :param user_id: User ID
        :return: User data dictionary
        """
        headers = {
            "Authorization": self.client.workspace_secret if self.client._workspace_secret else self.client.app_secret
        }
        path = f"/{self.client.app_id}/users/{user_id}"
        data = self.client.request("GET", path, headers=headers)
        return data or {}

    def list_users(
        self,
        id: Optional[str] = None,
        email: Optional[str] = None,
        phone: Optional[str] = None,
        created_before: Optional[str] = None,
        status: Optional[str] = None,  # "active" | "pending" | "inactive"
        page: Optional[int] = 1,
        limit: Optional[int] = 20
    ) -> Dict[str, Any]:
        """
        Lists users with optional filtering and pagination.

        :return: Dictionary with users list and pagination metadata
        :raises ValueError: If status is invalid
        """
        headers = {
            "Authorization": self.client.workspace_secret if self.client._workspace_secret else self.client.app_secret
        }
        params = {"page": page, "limit": limit}

        if id:
            params["id"] = id
        if email:
            params["email"] = email
        if phone:
            params["phone"] = phone
        if created_before:
            params["created_before"] = created_before
        if status:
            if status not in {"active", "pending", "inactive"}:
                raise ValueError("Invalid status. Must be 'active', 'pending', or 'inactive'.")
            params["status"] = status

        path = f"/{self.client.app_id}/users"
        return self.client.request("GET", path, headers=headers, params=params) or {}

    def list_all_users(
        self,
        email: Optional[str] = None,
        phone: Optional[str] = None,
        created_before: Optional[str] = None,
        status: Optional[str] = None,
        limit: int = 50
    ) -> Iterator[Dict[str, Any]]:
        """
        Generator that yields all users by automatically paginating.

        :yield: Individual user dictionaries
        """
        page = 1
        while True:
            response = self.list_users(
                email=email,
                phone=phone,
                created_before=created_before,
                status=status,
                page=page,
                limit=limit
            )
            users = response.get("users", [])
            if not users:
                break

            for user in users:
                yield user

            page += 1

    def delete_user(self, user_id: str) -> bool:
        """
        Deletes a user by ID.

        :param user_id: ID of the user to delete
        :return: True if deletion was successful
        :raises ValueError: If user_id or access_token is missing
        """
        if not user_id:
            raise ValueError("The 'user_id' is required.")

        headers = {
            "Authorization": self.client.workspace_secret if self.client._workspace_secret else self.client.app_secret
        }
        path = f"/{self.client.app_id}/users/{user_id}"
        response = self.client.request("DELETE", path, headers=headers)

        return response.get("message") == "ok" or response == {}



import logging
import requests
from typing import Optional, Dict, Any

from requests.adapters import HTTPAdapter
from urllib3 import Retry

from scute_client.exceptions import APIRequestError

logger = logging.getLogger(__name__)

class BaseClient:
    def __init__(
        self,
        workspace_id: str,
        workspace_secret: str,
        app_id: str,
        app_secret: str,
        base_url: str,
        api_version: str,
        timeout: int,
    ) -> None:

        self.workspace_id = workspace_id
        self._workspace_secret = workspace_secret
        self.workspace_secret = f"Bearer {workspace_secret}"
        self.app_id = app_id
        self._app_secret = app_secret
        self.app_secret = f"Bearer {app_secret}"
        self.base_url = base_url.rstrip("/")
        self.api_version = api_version
        self.timeout = timeout
        self.session = requests.Session()

        retries = Retry(
            total=3,
            backoff_factor=1,
            status_forcelist=[502, 503, 504],
            allowed_methods=["HEAD", "GET", "PUT", "POST", "DELETE", "OPTIONS", "PATCH"]
        )
        adapter = HTTPAdapter(max_retries=retries)
        self.session.mount("https://", adapter)

    def request(
            self,
            method: str,
            path: str,
            headers: Optional[Dict[str, str]] = None,
            params: Optional[Dict[str, Any]] = None,
            json: Optional[Dict[str, Any]] = None,
            data: Optional[Dict[str, Any]] = None,
            files: Optional[Dict[str, Any]] = None,
    ) -> Optional[Any]:
        url = f"{self.base_url}/{self.api_version}{path}"
        logger.debug(
            f"Scute request to {url} with method={method}, headers={headers}, "
            f"params={params}, json={json}, data={data}, files={files}"
        )

        try:
            response = self.session.request(
                method=method,
                url=url,
                headers=headers,
                params=params,
                json=json,
                data=data,
                files=files,
                timeout=self.timeout,
            )
        except requests.RequestException as e:
            logger.error(f"Request to {url} failed: {str(e)}")
            raise APIRequestError(status_code=-1, message=str(e))

        logger.debug(f"Scute response {response.status_code} from {url}: {response.text}")

        if response.status_code == 200:
            if response.content:
                try:
                    return response.json()
                except ValueError:
                    logger.warning(f"Invalid JSON from {url}")
                    return None
            return None

        raise APIRequestError(
            status_code=response.status_code,
            message=response.text,
            response=response,
        )



from typing import Dict, Any
from .base import BaseClient
from scute_client.exceptions import APIRequestError

class TokensAPI:
    def __init__(self, client: BaseClient):
        self.client = client

    def refresh_access_token(
        self,
        refresh_token: str,
        csrf_token: str
    ) -> Dict[str, Any]:
        """
        Refreshes the access token using the refresh and CSRF tokens.
        """
        headers = {
            "X-Refresh-Token": refresh_token,
            "X-CSRF-Token": csrf_token
        }
        path = f"/auth/{self.client.app_id}/tokens/refresh"
        return self.client.request("POST", path, headers=headers) or {}

    def verify_access_token(self, access_token: str) -> bool:
        """
        Verifies the validity of an access token.
        """
        headers = {"X-Authorization": access_token}
        path = f"/auth/{self.client.app_id}/tokens/access/verify"
        try:
            self.client.request("POST", path, headers=headers)
            return True
        except APIRequestError as e:
            if e.status_code == 401:
                return False
            raise

    def verify_refresh_token(self, refresh_token: str, csrf_token: str) -> bool:
        """
        Verifies the validity of a refresh token and CSRF token.
        """
        headers = {
            "X-Refresh-Token": refresh_token,
            "X-CSRF-Token": csrf_token
        }
        path = f"/auth/{self.client.app_id}/tokens/refresh/verify"
        try:
            self.client.request("POST", path, headers=headers)
            return True
        except APIRequestError as e:
            if e.status_code == 401:
                return False
            raise



from typing import Dict, Any
from .base import BaseClient

class AuthAPI:
    def __init__(self, client: BaseClient):
        self.client = client

    def get_current_user(self, access_token: str) -> Dict[str, Any]:
        headers = {"X-Authorization": access_token}
        path = f"/auth/{self.client.app_id}/current_user"
        data = self.client.request(method="GET", path=path, headers=headers)
        return data.get("user", {}) if data else {}

    def logout_current_user(self, access_token: str) -> bool:
        headers = {"X-Authorization": access_token}
        path = f"/auth/{self.client.app_id}/current_user/logout"
        self.client.request(method="POST", path=path, headers=headers)
        return True

    def create_app_session(self, access_token: str, app_id: str) -> Dict[str, Any]:
        headers = {
            "Authorization": self.client.workspace_secret,
            "X-Authorization": access_token,
        }
        files = {
            "app_id": (None, app_id),
        }  # multipart/form-data
        path = f"/workspaces/{self.client.workspace_id}/sessions"
        data = self.client.request(method="POST", path=path, headers=headers, files=files)
        return data.get("user", {}) if data else {}
